package cz.chladek.jpct.physicstest;

import com.threed.jpct.SimpleVector;
import com.threed.jpct.Object3D;
import com.threed.jpct.Matrix;

import com.bulletphysics.linearmath.MotionState;
import com.bulletphysics.linearmath.Transform;
import com.bulletphysics.linearmath.MatrixUtil;

public class JPCTBulletMotionState extends MotionState {

	public final Transform centerOfMassOffset = new Transform();
	private Object3D obj3d;
	private final float[] matDump;
	private final Matrix mat;

	public JPCTBulletMotionState(Object3D obj) {
		obj3d = obj;
		centerOfMassOffset.setIdentity();
		matDump = new float[16];
		mat = new Matrix();
	}

	public JPCTBulletMotionState(Object3D obj, Transform startTrans) {
		obj3d = obj;
		setGraphicFromTransform(startTrans);
		centerOfMassOffset.setIdentity();
		matDump = new float[16];
		mat = new Matrix();
	}

	public JPCTBulletMotionState(Object3D obj, Transform startTrans, Transform centerOfMassOffset) {
		obj3d = obj;
		setGraphicFromTransform(startTrans);
		this.centerOfMassOffset.set(centerOfMassOffset);
		matDump = new float[16];
		mat = new Matrix();
	}

	@Override
	public Transform getWorldTransform(Transform worldTrans) {
		setTransformFromGraphic(worldTrans);
		return worldTrans;
	}

	@Override
	public void setWorldTransform(Transform worldTrans) {
		setGraphicFromTransform(worldTrans);
	}

	private void setTransformFromGraphic(Transform tran) {
		SimpleVector p = SimpleVector.create();
		obj3d.getTransformedCenter(p);
		tran.origin.set(p.x, -p.y, -p.z);
		Matrix matrixGfx = obj3d.getRotationMatrix();
		matrixGfx.fillDump(matDump);
		MatrixUtil.getOpenGLSubMatrix(tran.basis, matDump);
	}

	private void setGraphicFromTransform(Transform tran) {
		SimpleVector pos = SimpleVector.create();
		obj3d.getTransformedCenter(pos);
		obj3d.translate(tran.origin.x - pos.x, -tran.origin.y - pos.y, -tran.origin.z - pos.z);

		Matrix matrix = obj3d.getRotationMatrix();
		matrix.fillDump(matDump);
		MatrixUtil.getOpenGLSubMatrix(tran.basis, matDump);

		mat.setDump(matDump);
		mat.rotateX((float) Math.PI);
		obj3d.setRotationMatrix(mat);
	}
}