package cz.chladek.jpct.physicstest;

import java.lang.reflect.Field;
import java.util.ArrayList;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import javax.vecmath.Vector3f;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.Window;
import android.view.WindowManager;
import android.view.WindowManager.LayoutParams;

import com.bulletphysics.collision.broadphase.AxisSweep3;
import com.bulletphysics.collision.dispatch.CollisionDispatcher;
import com.bulletphysics.collision.dispatch.DefaultCollisionConfiguration;
import com.bulletphysics.collision.shapes.BoxShape;
import com.bulletphysics.collision.shapes.CollisionShape;
import com.bulletphysics.collision.shapes.SphereShape;
import com.bulletphysics.dynamics.DiscreteDynamicsWorld;
import com.bulletphysics.dynamics.RigidBody;
import com.bulletphysics.dynamics.RigidBodyConstructionInfo;
import com.bulletphysics.dynamics.constraintsolver.SequentialImpulseConstraintSolver;
import com.bulletphysics.linearmath.Clock;
import com.bulletphysics.linearmath.DefaultMotionState;
import com.bulletphysics.linearmath.Transform;
import com.threed.jpct.Camera;
import com.threed.jpct.Config;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Interact2D;
import com.threed.jpct.Light;
import com.threed.jpct.Loader;
import com.threed.jpct.Logger;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.util.AAConfigChooser;
import com.threed.jpct.util.MemoryHelper;

import cz.chladek.jpct.physicstest.PerformanceMonitor.PerformanceRecord;

public class PhysicsTestActivity extends Activity {

	private static PhysicsTestActivity master = null;
	private GLSurfaceView mGLView;
	private MyRenderer renderer = null;
	private FrameBuffer fb = null;
	private World world = null;
	private final Ticker ticker = new Ticker(15);
	private Renderer2D renderer2D;
	private PerformanceMonitor monitor;

	private int screenWidth;
	private int screenHeight;

	private float touchTurn;
	private float touchTurnUp;

	private float xpos = -1;
	private float ypos = -1;
	private float xpos1 = -1;
	private float ypos1 = -1;

	private Light light;

	private float cameraDistance = 40;
	private float cameraAngle = 1.18f;
	private float cameraHeight = -0.4f;
	private float rotatePower;
	private float turnPower;
	private float zoomPower = 1;
	private SimpleVector cameraLookAt;

	private long nextBoxTime, touchDownTime;
	private float firstX;
	private float firstY;

	private boolean interact;
	private Object3D interactBody;
	private boolean debug = true;

	private boolean cameraZoom;
	private float zoomValueLast;
	private float zoomValue;
	private boolean touchUseable = true;

	// ----------------- physics -------------------

	private Object3D sBox;
	private Object3D sphere;

	public int numBoxesX = 3;
	public int numBoxesY = 3;
	public int numBoxesZ = 3;

	public DiscreteDynamicsWorld dynamicWorld;
	public int maxSubSteps;
	public float timeStep, fixedTimeStep;
	protected Clock clock;
	private CollisionDispatcher dispatcher;
	private DefaultCollisionConfiguration collisionConfiguration;

	private ArrayList<RigidBody> bodyList;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Logger.log("onCreate");

		clock = new Clock();
		monitor = new PerformanceMonitor(10000, 250);
		monitor.start();

		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE);

		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(LayoutParams.FLAG_FULLSCREEN, LayoutParams.FLAG_FULLSCREEN);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON, WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

		if (master != null)
			copy(master);

		super.onCreate(savedInstanceState);
		mGLView = new GLSurfaceView(getApplication());

		mGLView.setEGLContextClientVersion(2);
		mGLView.setEGLConfigChooser(new AAConfigChooser(mGLView));

		renderer = new MyRenderer();
		mGLView.setRenderer(renderer);
		setContentView(mGLView);
	}

	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
	}

	@Override
	protected void onPause() {
		super.onPause();
		mGLView.onPause();
	}

	@Override
	protected void onResume() {
		super.onResume();
		mGLView.onResume();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	private void copy(Object src) {
		try {
			Logger.log("Copying data from master Activity!");
			Field[] fs = src.getClass().getDeclaredFields();
			for (Field f : fs) {
				f.setAccessible(true);
				f.set(this, f.get(src));
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent me) {

		if (me.getAction() == MotionEvent.ACTION_DOWN) {
			touchUseable = true;
			xpos = me.getX();
			ypos = me.getY();
			firstX = xpos;
			firstY = ypos;
			touchDownTime = System.currentTimeMillis();
		}

		if (me.getActionMasked() == MotionEvent.ACTION_POINTER_DOWN) {
			cameraZoom = true;
			touchUseable = false;
			xpos1 = me.getX(1);
			ypos1 = me.getY(1);
			zoomValueLast = zoomValue();
			touchTurn = 0;
			touchTurnUp = 0;
		}

		if (me.getActionMasked() == MotionEvent.ACTION_POINTER_UP) {
			cameraZoom = false;
			xpos1 = -1;
			ypos1 = -1;
		}

		if (me.getAction() == MotionEvent.ACTION_UP) {
			cameraZoom = false;
			touchUseable = false;
			interact = false;
			interactBody = null;
			xpos = -1;
			ypos = -1;
			touchTurn = 0;
			touchTurnUp = 0;

			float xd = me.getX() - firstX;
			float yd = me.getY() - firstY;
			if (xd < 5 && xd > -5 && yd < 5 && yd > -5 && !interact && System.currentTimeMillis() - nextBoxTime >= 100) {
				Camera cam = world.getCamera();
				SimpleVector dir = Interact2D.reproject2D3DWS(cam, fb, (int) me.getX(), (int) me.getY());
				dir.scalarMul(-40);
				RigidBody body = renderer.addSphere(cam.getPosition());
				Vector3f force = new Vector3f(-dir.x, dir.y, dir.z);
				body.activate(true);
				body.setLinearVelocity(force);
				nextBoxTime = System.currentTimeMillis();
			}
			return true;
		}

		if (me.getAction() == MotionEvent.ACTION_MOVE) {
			float xd = me.getX() - xpos;
			float yd = me.getY() - ypos;
			xpos = me.getX();
			ypos = me.getY();

			if (cameraZoom && !touchUseable) {
				xpos1 = me.getX(1);
				ypos1 = me.getY(1);
				float currentZoomValue = zoomValue();
				float newValue = 1 + (zoomValueLast - currentZoomValue) / 400f;
				zoomValueLast = currentZoomValue;
				zoomValue = newValue;
			} else if (!interact && !cameraZoom && touchUseable) {
				touchTurn = xd / -520f;
				touchTurnUp = yd / -520f;
			}
			return true;
		}

		try {
			Thread.sleep(15);
		} catch (Exception e) {
		}

		return super.onTouchEvent(me);
	}

	private float zoomValue() {
		float distance = (float) Math.sqrt((xpos - xpos1) * (xpos - xpos1) + (ypos - ypos1) * (ypos - ypos1));
		return distance;
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent msg) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			int pid = android.os.Process.myPid();
			android.os.Process.killProcess(pid);
			return true;
		}

		if (keyCode == KeyEvent.KEYCODE_MENU) {
			debug = (debug) ? false : true;
			for (RigidBody box : bodyList)
				((Object3D) box.getUserPointer()).setTransparency(-1);
			return true;
		}
		return super.onKeyDown(keyCode, msg);
	}

	protected boolean isFullscreenOpaque() {
		return true;
	}

	class MyRenderer implements GLSurfaceView.Renderer {

		private final Resources res = getResources();
		private long timeForPhysics;
		private int fps, lfps;
		private long time = System.currentTimeMillis();

		private float physicsTime;
		private int calcPhysicsCount;
		private boolean checkTime = true;
		private boolean allActive;

		public MyRenderer() {
			Config.glTransparencyOffset = 0;
			Config.glTransparencyMul = 0.01f;
		}

		@Override
		public void onSurfaceChanged(GL10 gl, int w, int h) {
			if (fb != null)
				fb.dispose();

			fb = new FrameBuffer(w, h);

			screenWidth = w;
			screenHeight = h;

			renderer2D = new Renderer2D(fb);

			if (master == null) {

				bodyList = new ArrayList<RigidBody>();

				Texture texture = new Texture(res.openRawResource(R.raw.box));
				TextureManager.getInstance().addTexture("BOX.JPG", texture);

				Texture texture2 = new Texture(res.openRawResource(R.raw.font));
				texture2.setFiltering(false);
				TextureManager.getInstance().addTexture("gui_font", texture2);

				Texture texture3 = new Texture(res.openRawResource(R.raw.back));
				TextureManager.getInstance().addTexture("back", texture3);

				world = new World();

				sBox = Loader.loadSerializedObject(res.openRawResource(R.raw.boxmodel));
				sBox.build();
				sphere = Primitives.getSphere(1.5f);
				sphere.calcTextureWrapSpherical();
				sphere.setSpecularLighting(Object3D.SPECULAR_ENABLED);
				sphere.setTexture("BOX.JPG");

				Object3D ground = Primitives.getPlane(20, 5);
				ground.rotateX((float) Math.PI / 2);
				ground.setTexture("back");
				ground.build();
				world.addObject(ground);
				cameraLookAt = new SimpleVector();

				light = new Light(world);
				light.setPosition(new SimpleVector(0, -20, -20));
				light.setIntensity(200, 200, 200);

				collisionConfiguration = new DefaultCollisionConfiguration();
				dispatcher = new CollisionDispatcher(collisionConfiguration);
				Vector3f worldAabbMin = new Vector3f(Float.MIN_VALUE, Float.MIN_VALUE, Float.MIN_VALUE);
				Vector3f worldAabbMax = new Vector3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
				AxisSweep3 overlappingPairCache = new AxisSweep3(worldAabbMin, worldAabbMax);
				SequentialImpulseConstraintSolver solver = new SequentialImpulseConstraintSolver();

				dynamicWorld = new DiscreteDynamicsWorld(dispatcher, overlappingPairCache, solver, collisionConfiguration);
				dynamicWorld.setGravity(new Vector3f(0, -10, 0));
				dynamicWorld.getDispatchInfo().allowedCcdPenetration = 0f;

				CollisionShape groundShape = new BoxShape(new Vector3f(50, 50, 50));
				Transform groundTransform = new Transform();
				groundTransform.setIdentity();
				groundTransform.origin.set(new Vector3f(0.f, -50.f, 0.f));
				float mass = 0f;
				Vector3f localInertia = new Vector3f(0, 0, 0);
				DefaultMotionState myMotionState = new DefaultMotionState(groundTransform);
				RigidBodyConstructionInfo rbInfo = new RigidBodyConstructionInfo(mass, myMotionState, groundShape, localInertia);
				RigidBody body = new RigidBody(rbInfo);
				dynamicWorld.addRigidBody(body);

				dynamicWorld.clearForces();

				for (int y = -2; y > -numBoxesX * 4; y -= 4)
					for (int x = -2 * numBoxesY; x < numBoxesY * 2; x += 4)
						for (int z = -2 * numBoxesZ; z < numBoxesZ * 2; z += 4)
							addBox(x, y - 8, z);

				System.runFinalization();
				System.gc();
				MemoryHelper.compact();

				if (master == null) {
					Logger.log("Saving master Activity!");
					master = PhysicsTestActivity.this;
				}
			}
		}

		public RigidBody addSphere(SimpleVector position) {
			SphereShape shape = new SphereShape(1.5f);
			float mass = 12;
			Vector3f localInertia = new Vector3f(0, 0, 0);
			shape.calculateLocalInertia(mass, localInertia);

			Object3D spheregfx = new Object3D(sphere);

			spheregfx.translate(position);
			spheregfx.build();
			spheregfx.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
			spheregfx.setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);

			world.addObject(spheregfx);

			JPCTBulletMotionState ms = new JPCTBulletMotionState(spheregfx);

			RigidBodyConstructionInfo rbInfo = new RigidBodyConstructionInfo(mass, ms, shape, localInertia);
			RigidBody body = new RigidBody(rbInfo);
			body.setRestitution(0.5f);
			body.setFriction(0.5f);
			body.setDamping(0f, 0.2f);

			body.setUserPointer(spheregfx);
			spheregfx.setUserObject(body);
			bodyList.add(body);

			dynamicWorld.addRigidBody(body);

			return body;
		}

		public void addBox(int x, int y, int z) {
			BoxShape shape = new BoxShape(new Vector3f(2, 2, 2));
			float mass = 20;
			Vector3f localInertia = new Vector3f(0, 0, 0);
			shape.calculateLocalInertia(mass, localInertia);

			Object3D boxgfx = new Object3D(sBox);
			boxgfx.translate(x, y, z);
			boxgfx.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
			boxgfx.setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);
			boxgfx.build();
			world.addObject(boxgfx);

			JPCTBulletMotionState ms = new JPCTBulletMotionState(boxgfx);

			RigidBodyConstructionInfo rbInfo = new RigidBodyConstructionInfo(mass, ms, shape, localInertia);
			RigidBody body = new RigidBody(rbInfo);
			body.setRestitution(0.1f);
			body.setFriction(0.5f);
			body.setDamping(0, 0);

			body.setUserPointer(boxgfx);
			boxgfx.setUserObject(body);
			bodyList.add(body);

			dynamicWorld.addRigidBody(body);
		}

		@Override
		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		}

		@Override
		public void onDrawFrame(GL10 gl) {
			try {
				long ticks = ticker.getTicks();

				calcCameraPosition(ticks);
				if (!interact)
					setCameraPosition();
				world.getCamera().lookAt(cameraLookAt);

				refreshPhysics();
				if (debug)
					checkActiveBodies(ticks);

				if (checkTime) {
					physicsTime += timeForPhysics;
					calcPhysicsCount++;
					if (!allActive) {
						physicsTime = physicsTime / calcPhysicsCount;
						checkTime = false;
					}
				}

				checkInteraction();

				fb.clear();
				world.renderScene(fb);
				world.draw(fb);

				if (debug) {
					String fpsText = "fps " + lfps + (!checkTime ? " | jBullet " + physicsTime + "ms" : "");
					int textWidth = Renderer2D.getTextWidth(fpsText) + 5;
					renderer2D.blitText(fpsText, screenWidth - textWidth, 5);
					renderer2D.blitText("Time to calculate physics " + timeForPhysics + "ms", 5, 5);
					showResourcesUsage();
				}

				fb.display();

				if (System.currentTimeMillis() - time >= 1000) {
					lfps = (fps + lfps) >> 1;
					fps = 0;
					time = System.currentTimeMillis();
				}
				fps++;
			} catch (Exception e) {
				Logger.log(e, Logger.MESSAGE);
			}
		}

		private void checkInteraction() {
			if (interact) {
				RigidBody body = (RigidBody) interactBody.getUserObject();
				body.activate(true);

				SimpleVector screenPos = Interact2D.projectCenter3D2D(world.getCamera(), fb, interactBody);
				SimpleVector velocity = new SimpleVector(screenPos.x - xpos, screenPos.y - ypos, 0);
				velocity.rotateY(cameraAngle + (float) Math.PI / 2);

				if (velocity.y < 0 && interactBody.getTransformedCenter().y >= -4f)
					velocity.y = 0;

				body.setLinearVelocity(new Vector3f(-velocity.x, velocity.y, velocity.z));
			} else if (!cameraZoom && touchUseable) {
				int xd = (int) (xpos - firstX);
				int yd = (int) (ypos - firstY);
				if (xd < 5 && xd > -5 && yd < 5 && yd > -5 && System.currentTimeMillis() - touchDownTime >= 500) {
					Camera cam = world.getCamera();
					SimpleVector dir = Interact2D.reproject2D3DWS(cam, fb, (int) xpos, (int) ypos);
					Object[] values = world.calcMinDistanceAndObject3D(cam.getPosition(), dir, Float.MAX_VALUE);
					interactBody = (Object3D) values[1];
					if (interactBody != null)
						interact = true;
				}
			}
		}

		private void calcCameraPosition(long ticks) {
			// up/down
			if (touchTurnUp != 0) {
				touchTurnUp *= ticks;
				if (cameraHeight <= -0.1f && cameraHeight >= -1f)
					cameraHeight += touchTurnUp;
				turnPower = touchTurnUp;
				touchTurnUp = 0;
			} else {
				turnPower *= 0.95f;
				cameraHeight += turnPower;
			}
			if (cameraHeight > -0.1f)
				cameraHeight = -0.1f;
			else if (cameraHeight < -1f)
				cameraHeight = -1f;

			// right/left
			if (touchTurn != 0) {
				rotatePower = touchTurn * ticks;
				touchTurn = 0;
			} else
				rotatePower *= 0.95f;

			// zoom
			if (cameraZoom)
				zoomPower = zoomValue;
			else
				zoomPower = 1 + (zoomPower - 1) * 0.9f;
		}

		private void setCameraPosition() {
			Camera cam = world.getCamera();
			cameraAngle += rotatePower;
			float x = (float) Math.cos(cameraAngle);
			float z = (float) Math.sin(cameraAngle);
			SimpleVector camPosition = SimpleVector.create(x, cameraHeight, z);
			float newDistance = cameraDistance * zoomPower;
			if (newDistance > 10 && newDistance < 60)
				cameraDistance = newDistance;
			camPosition.scalarMul(cameraDistance);
			cam.setPosition(camPosition);
		}

		private void checkActiveBodies(long ticks) {
			boolean active = false;
			for (RigidBody box : bodyList) {
				Object3D object = (Object3D) box.getUserPointer();
				if (box.isActive()) {
					object.setTransparency(-1);
					active = true;
				} else {
					if (object.getTransparency() == -1)
						object.setTransparency(100);
					int tr = object.getTransparency();
					tr -= (int) (7 * ticks);
					if (tr < 40)
						tr = 40;
					object.setTransparency(tr);
				}
			}
			allActive = active;
		}

		private void refreshPhysics() {
			float ms = clock.getTimeMicroseconds();
			clock.reset();
			timeForPhysics = System.currentTimeMillis();
			dynamicWorld.stepSimulation(ms / 1000000f);
			timeForPhysics = System.currentTimeMillis() - timeForPhysics;
		}
	}

	private void showResourcesUsage() {
		int y = screenHeight - 20;
		int coreCount = monitor.getCoreCount();
		PerformanceRecord record = null;
		while (monitor.hasMoreRecords()) {
			record = monitor.getNextRecord();
			int recNumb = record.getRecordNumb() * 3;
			int memUsage = record.getMemoryUsage();
			renderer2D.blitFilledBox(fb, 265 + recNumb, y - memUsage, 2, memUsage, -1, RGBColor.WHITE);
			for (int i = 0; i < coreCount; i++) {
				int xShift = i * 150;
				int coreLoad = record.getCoreLoad(i);
				renderer2D.blitFilledBox(fb, 415 + xShift + recNumb, y - coreLoad, 2, coreLoad, -1, RGBColor.RED);
			}
		}
		String text = "VM M - " + record.getMemoryUsageMean() + "%";
		int width = Renderer2D.getTextWidth(text);
		renderer2D.blitText(text, 325 - width / 2, y, 100);
		for (int i = 0; i < coreCount; i++) {
			text = "CPU" + i + " - " + record.getCoreLoadMean(i) + "%";
			width = Renderer2D.getTextWidth(text);
			renderer2D.blitText(text, i * 150 + 475 - width / 2, y, 100);
		}
	}
}