package cz.chladek.jpct.physicstest;

import com.threed.jpct.FrameBuffer;
import com.threed.jpct.RGBColor;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;

public class Renderer2D {
	private FrameBuffer fb;
	private TextureManager tm = TextureManager.getInstance();

	private static final Texture REFERENCE_POINT = new Texture(8, 8, RGBColor.WHITE);

	private static int charHeight = 16;
	private static int charWidth = 9;
	private static int charPerLine = 32;

	public Renderer2D(FrameBuffer fb) {
		this.fb = fb;
	}

	public static int getTextWidth(String text) {
		int length = text.length() * charWidth;
		return length;
	}

	public void blitText(String text, int x, int y) {
		Texture font = tm.getTexture("gui_font");
		for (int i = 0; i < text.length(); i++) {
			char ch = text.charAt(i);
			int yS = ch / charPerLine;
			int xS = (ch % charPerLine == 0) ? 0 : ch - yS * charPerLine;
			fb.blit(font, xS * charWidth, yS * charHeight, x, y, charWidth, charHeight, FrameBuffer.TRANSPARENT_BLITTING);
			x += charWidth;
		}
	}

	public void blitText(String text, int x, int y, RGBColor addColor) {
		Texture font = tm.getTexture("gui_font");
		for (int i = 0; i < text.length(); i++) {
			char ch = text.charAt(i);
			int yS = ch / charPerLine;
			int xS = (ch % charPerLine == 0) ? 0 : ch - yS * charPerLine;
			fb.blit(font, xS * charWidth, yS * charHeight, x, y, charWidth, charHeight, charWidth, charHeight, 100,
					FrameBuffer.TRANSPARENT_BLITTING, addColor);
			x += charWidth;
		}
	}

	public void blitText(String text, int x, int y, int transparency) {
		Texture font = tm.getTexture("gui_font");
		for (int i = 0; i < text.length(); i++) {
			char ch = text.charAt(i);
			int yS = ch / charPerLine;
			int xS = (ch % charPerLine == 0) ? 0 : ch - yS * charPerLine;
			fb.blit(font, xS * charWidth, yS * charHeight, x, y, charWidth, charHeight, charWidth, charHeight, transparency,
					FrameBuffer.TRANSPARENT_BLITTING, null);
			x += charWidth;
		}
	}

	public void blitFilledBox(FrameBuffer fb, int x, int y, int w, int h, int transparency, RGBColor color) {
		fb.blit(REFERENCE_POINT, 0, 0, x, y, 8, 8, w, h, transparency, false, color);
	}
}